@@ -30,11 +30,15 @@
 #include <KDebug>
 
 #include <KPixmapSequence>
+#include <KConfigDialogManager>
 
 ClassListWidget::ClassListWidget(QWidget *parent) :
     QListView(parent),
-    m_request(0)
+    m_request(0),
+    m_showClasses(false)
 {
+    KConfigDialogManager::changedMap()->insert("ClassListWidget", SIGNAL(changed(QString)));
+
     m_model = new QStandardItemModel(this);
     setModel(m_model);
     setItemDelegate(new NoSelectionRectDelegate(this));
@@ -47,19 +51,21 @@ ClassListWidget::ClassListWidget(QWidget *parent) :
 
     connect(m_model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
             this, SLOT(modelChanged()));
+
+    m_delayedInit.setInterval(0);
+    m_delayedInit.setSingleShot(true);
+    connect(&m_delayedInit, SIGNAL(timeout()), SLOT(init()));
+    m_delayedInit.start();
 }
 
 ClassListWidget::~ClassListWidget()
 {
 }
 
-void ClassListWidget::reload(const QString &reqDestName, const QStringList &memberNames)
+void ClassListWidget::init()
 {
-    // If we have an old request running discard it's result and get a new one
-    if (m_request) {
-        connect(m_request, SIGNAL(finished()), this, SLOT(deleteLater()));
-        disconnect(m_request, SIGNAL(finished()), this, SLOT(loadFinished()));
-    }
+    m_busySeq->start(); // Start spining
+    m_model->clear();
 
     QStringList att;
     att << KCUPS_PRINTER_NAME;
@@ -67,74 +73,79 @@ void ClassListWidget::reload(const QString &reqDestName, const QStringList &memb
     // Get destinations with these masks
     m_request = new KCupsRequest;
     connect(m_request, SIGNAL(finished()), this, SLOT(loadFinished()));
-    m_request->setProperty("reqDestName", reqDestName);
-    m_request->setProperty("memberNames", memberNames);
-    m_request->getPrinters(att,
-                           CUPS_PRINTER_CLASS | CUPS_PRINTER_REMOTE | CUPS_PRINTER_IMPLICIT);
-
-    m_busySeq->start(); // Start spining
+    if (m_showClasses) {
+        m_request->getPrinters(att);
+    } else {
+        m_request->getPrinters(att,
+                               CUPS_PRINTER_CLASS | CUPS_PRINTER_REMOTE | CUPS_PRINTER_IMPLICIT);
+    }
 }
 
 void ClassListWidget::loadFinished()
 {
+    // If we have an old request running discard it's result and get a new one
+    if (m_request != sender()) {
+        sender()->deleteLater();
+        return;
+    }
+
     m_busySeq->stop(); // Stop spining
 
-    KCupsPrinters printers;
-    QString reqDestName;
-    QStringList memberNames;
-    printers    = m_request->printers();
-    reqDestName = m_request->property("reqDestName").toString();
-    memberNames = m_request->property("memberNames").toStringList();
+    KCupsPrinters printers = m_request->printers();
     m_request->deleteLater();
     m_request = 0;
 
-    m_model->clear();
-    QStringList origMemberUris;
-    foreach (const QString &memberUri, memberNames) {
-        foreach (const KCupsPrinter &printer, printers) {
-            if (printer.name() == memberUri) {
-                origMemberUris << printer.uriSupported();
-                break;
-            }
-        }
-    }
-    m_model->setProperty("orig-member-uris", origMemberUris);
-    m_selectedDests = origMemberUris;
-
     foreach (const KCupsPrinter &printer, printers) {
         QString destName = printer.name();
-        if (destName != reqDestName) {
-            QStandardItem *item = new QStandardItem(destName);
+        if (destName != m_printerName) {
+            QStandardItem *item = new QStandardItem;
+            item->setText(destName);
             item->setCheckable(true);
             item->setEditable(false);
-            if (memberNames.contains(destName)) {
-                item->setCheckState(Qt::Checked);
-            }
             item->setData(printer.uriSupported());
+            updateItemState(item);
+
             m_model->appendRow(item);
         }
     }
 
-    // clear old values
-    m_changed = false;
+    modelChanged();
 }
 
 void ClassListWidget::modelChanged()
+{
+    QStringList currentMembers = currentSelected(false);
+
+    m_changed = m_selectedPrinters != currentMembers;
+
+    emit changed(selectedPrinters());
+    emit changed(m_changed);
+}
+
+QStringList ClassListWidget::currentSelected(bool uri) const
 {
     QStringList currentMembers;
     for (int i = 0; i < m_model->rowCount(); i++) {
         QStandardItem *item = m_model->item(i);
         if (item && item->checkState() == Qt::Checked) {
-            currentMembers << item->data().toString();
+            if (uri) {
+                currentMembers << item->data().toString();
+            } else {
+                currentMembers << item->text();
+            }
         }
     }
     currentMembers.sort();
+    return currentMembers;
+}
 
-    // store the new values
-    m_selectedDests = currentMembers;
-
-    m_changed = m_model->property("orig-member-uris").toStringList() != currentMembers;
-    emit changed(m_changed);
+void ClassListWidget::updateItemState(QStandardItem *item) const
+{
+    if (m_selectedPrinters.contains(item->text())) {
+        item->setCheckState(Qt::Checked);
+    } else {
+        item->setCheckState(Qt::Unchecked);
+    }
 }
 
 bool ClassListWidget::hasChanges()
@@ -142,9 +153,37 @@ bool ClassListWidget::hasChanges()
     return m_changed;
 }
 
-QStringList ClassListWidget::selectedDests() const
+void ClassListWidget::setPrinter(const QString &printer)
 {
-    return m_selectedDests;
+    if (m_printerName != printer) {
+        m_printerName = printer;
+        m_delayedInit.start();
+    }
+}
+
+QString ClassListWidget::selectedPrinters() const
+{
+    return currentSelected(false).join(QLatin1String("|"));
+}
+
+void ClassListWidget::setSelectedPrinters(const QString &selected)
+{
+    m_selectedPrinters = selected.split(QLatin1Char('|'));
+    m_selectedPrinters.sort();
+    m_delayedInit.start();
+}
+
+bool ClassListWidget::showClasses() const
+{
+    return m_showClasses;
+}
+
+void ClassListWidget::setShowClasses(bool enable)
+{
+    if (m_showClasses != enable) {
+        m_showClasses = enable;
+        m_delayedInit.start();
+    }
 }
 
 #include "ClassListWidget.moc"
