@@ -36,8 +36,9 @@ public:
 
     int id() const;
     QString idStr() const;
-    QString name() const;
-    QString ownerName() const;
+    QString name() const;    
+    QString originatingUserName() const;
+    QString originatingHostName() const;
     QString printer() const;
     QDateTime createdAt() const;
     QDateTime completedAt() const;
@@ -45,15 +46,17 @@ public:
     int pages() const;
     int processedPages() const;
     int size() const;
+    bool preserved() const;
 
     static QString iconName(ipp_jstate_t state);
     ipp_jstate_t state() const;
     QString stateMsg() const;
 
+    bool reprintEnabled() const;
+
     static bool cancelEnabled(ipp_jstate_t state);
     static bool holdEnabled(ipp_jstate_t state);
     static bool releaseEnabled(ipp_jstate_t state);
-    static bool restartEnabled(ipp_jstate_t state);
 
 protected:
     KCupsJob(const QVariantHash &arguments);
